@@ -14,6 +14,7 @@
 #include <glib-object.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
 
 #include "process-tree-view.h"
 #include "task-manager.h"
@@ -55,7 +56,11 @@ struct _XtmProcessTreeView
 };
 G_DEFINE_TYPE (XtmProcessTreeView, xtm_process_tree_view, GTK_TYPE_TREE_VIEW)
 
+static void		xtm_process_tree_view_finalize			(GObject *object);
+
 static gboolean		treeview_clicked				(XtmProcessTreeView *treeview, GdkEventButton *event);
+static gboolean		treeview_key_pressed				(XtmProcessTreeView *treeview, GdkEventKey *event);
+static void		column_task_pack_cells				(XtmProcessTreeView *treeview, GtkTreeViewColumn *column);
 static void		columns_changed					(XtmProcessTreeView *treeview);
 static void		read_columns_positions				(XtmProcessTreeView *treeview);
 static void		save_columns_positions				(XtmProcessTreeView *treeview);
@@ -69,13 +74,15 @@ static void		settings_changed				(GObject *object, GParamSpec *pspec, XtmProcess
 static void
 xtm_process_tree_view_class_init (XtmProcessTreeViewClass *klass)
 {
+	GObjectClass *class = G_OBJECT_CLASS (klass);
 	xtm_process_tree_view_parent_class = g_type_class_peek_parent (klass);
+	class->finalize = xtm_process_tree_view_finalize;
 }
 
 static void
 xtm_process_tree_view_init (XtmProcessTreeView *treeview)
 {
-	GtkCellRenderer *cell_text, *cell_right_aligned, *cell_cmdline;
+	GtkCellRenderer *cell_text, *cell_right_aligned;
 	GtkTreeViewColumn *column;
 	gboolean visible;
 
@@ -85,11 +92,16 @@ xtm_process_tree_view_init (XtmProcessTreeView *treeview)
 	{
 		gchar *uid_name;
 		get_owner_uid (&treeview->owner_uid, &uid_name);
+		g_free (uid_name);
 		g_object_get (treeview->settings, "show-all-processes", &treeview->show_all_processes_cached, NULL);
 	}
 
 	/* Create tree view model */
+#ifdef HAVE_WNCK
+	treeview->model = gtk_list_store_new (XTM_PTV_N_COLUMNS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_UINT64,
+#else
 	treeview->model = gtk_list_store_new (XTM_PTV_N_COLUMNS, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_UINT64,
+#endif
 		G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_UINT, G_TYPE_STRING, G_TYPE_FLOAT, G_TYPE_STRING, G_TYPE_INT,
 		G_TYPE_STRING, G_TYPE_STRING, G_TYPE_LONG);
 
@@ -104,15 +116,14 @@ xtm_process_tree_view_init (XtmProcessTreeView *treeview)
 	cell_right_aligned = gtk_cell_renderer_text_new ();
 	g_object_set (cell_right_aligned, "xalign", 1.0, NULL);
 
-	cell_cmdline = gtk_cell_renderer_text_new ();
-	g_object_set (cell_cmdline, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
-
 	/* Retrieve initial tree view columns positions */
 	read_columns_positions (treeview);
 
 	/* Create tree view columns */
 #define COLUMN_PROPERTIES "expand", TRUE, "clickable", TRUE, "reorderable", TRUE, "resizable", TRUE, "visible", TRUE
-	column = gtk_tree_view_column_new_with_attributes (_("Task"), cell_cmdline, "text", XTM_PTV_COLUMN_COMMAND, "cell-background", XTM_PTV_COLUMN_BACKGROUND, "foreground", XTM_PTV_COLUMN_FOREGROUND, NULL);
+	column = gtk_tree_view_column_new ();
+	gtk_tree_view_column_set_title (GTK_TREE_VIEW_COLUMN (column), _("Task"));
+	column_task_pack_cells (treeview, column);
 	g_object_set (column, COLUMN_PROPERTIES, NULL);
 	g_object_set_data (G_OBJECT (column), "sort-column-id", GINT_TO_POINTER (XTM_PTV_COLUMN_COMMAND));
 	g_object_set_data (G_OBJECT (column), "column-id", GINT_TO_POINTER (COLUMN_COMMAND));
@@ -203,12 +214,60 @@ xtm_process_tree_view_init (XtmProcessTreeView *treeview)
 	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (treeview), (GtkTreeViewSearchEqualFunc)search_func, NULL, NULL);
 	g_signal_connect (treeview, "columns-changed", G_CALLBACK (columns_changed), NULL);
 	g_signal_connect (treeview, "button-press-event", G_CALLBACK (treeview_clicked), NULL);
+	g_signal_connect (treeview, "key-press-event", G_CALLBACK (treeview_key_pressed), NULL);
+}
+
+static void
+xtm_process_tree_view_finalize (GObject *object)
+{
+	XtmProcessTreeView *treeview = XTM_PROCESS_TREE_VIEW (object);
+
+	if (GTK_IS_TREE_MODEL (treeview->model))
+	{
+		g_object_unref (treeview->model);
+		treeview->model = NULL;
+	}
+
+	if (GTK_IS_TREE_MODEL (treeview->model_filter))
+	{
+		g_object_unref (treeview->model_filter);
+		treeview->model_filter = NULL;
+	}
+
+	if (XTM_IS_SETTINGS (treeview->settings))
+	{
+		g_object_unref (treeview->settings);
+	}
+
+	G_OBJECT_CLASS (xtm_process_tree_view_parent_class)->finalize (object);
 }
 
 /**
  * Helper functions
  */
 
+static void
+column_task_pack_cells (XtmProcessTreeView *treeview, GtkTreeViewColumn *column)
+{
+	GtkCellRenderer *cell_cmdline, *cell_icon;
+	gboolean show_application_icons;
+
+	g_object_get (treeview->settings, "show-application-icons", &show_application_icons, NULL);
+	if (show_application_icons)
+	{
+#ifdef HAVE_WNCK
+		cell_icon = gtk_cell_renderer_pixbuf_new ();
+		gtk_tree_view_column_pack_start (GTK_TREE_VIEW_COLUMN (column), cell_icon, FALSE);
+		gtk_tree_view_column_set_attributes (GTK_TREE_VIEW_COLUMN (column), cell_icon, "pixbuf", XTM_PTV_COLUMN_ICON, "cell-background", XTM_PTV_COLUMN_BACKGROUND, NULL);
+#endif
+	}
+
+	cell_cmdline = gtk_cell_renderer_text_new ();
+	g_object_set (cell_cmdline, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+	gtk_tree_view_column_pack_start (GTK_TREE_VIEW_COLUMN (column), cell_cmdline, TRUE);
+	gtk_tree_view_column_set_attributes (GTK_TREE_VIEW_COLUMN (column), cell_cmdline, "text", XTM_PTV_COLUMN_COMMAND, "cell-background", XTM_PTV_COLUMN_BACKGROUND, "foreground", XTM_PTV_COLUMN_FOREGROUND, NULL);
+}
+
 static void
 columns_changed (XtmProcessTreeView *treeview)
 {
@@ -218,14 +277,20 @@ columns_changed (XtmProcessTreeView *treeview)
 	gint i;
 
 	columns = gtk_tree_view_get_columns (GTK_TREE_VIEW (treeview));
+	if (g_list_length (columns) < N_COLUMNS)
+	{
+		g_list_free (columns);
+		return;
+	}
+
 	for (l = columns, i = 0; l != NULL; l = l->next, i++)
 	{
 		column = GTK_TREE_VIEW_COLUMN (l->data);
 		column_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (column), "column-id"));
 		treeview->columns_positions[column_id] = i;
 	}
-	g_list_free (columns);
 
+	g_list_free (columns);
 	save_columns_positions (treeview);
 }
 
@@ -270,11 +335,17 @@ save_columns_positions (XtmProcessTreeView *treeview)
 static void
 cb_send_signal (GtkMenuItem *mi, gpointer user_data)
 {
+	XtmSettings *settings;
+	gboolean prompt_terminate_task;
 	GtkWidget *dialog;
 	guint pid = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (mi), "pid"));
 	gint xtm_signal = GPOINTER_TO_INT (user_data);
 
-	if (xtm_signal == XTM_SIGNAL_TERMINATE || xtm_signal == XTM_SIGNAL_KILL)
+	settings = xtm_settings_get_default ();
+	g_object_get (settings, "prompt-terminate-task", &prompt_terminate_task, NULL);
+	g_object_unref (settings);
+
+	if ((xtm_signal == XTM_SIGNAL_TERMINATE && prompt_terminate_task) || xtm_signal == XTM_SIGNAL_KILL)
 	{
 		gint res;
 		dialog = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
@@ -282,6 +353,7 @@ cb_send_signal (GtkMenuItem *mi, gpointer user_data)
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 			_("Are you sure you want to send a signal to the PID %d?"), pid);
 		gtk_window_set_title (GTK_WINDOW (dialog), _("Task Manager"));
+		gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
 		res = gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 		if (res != GTK_RESPONSE_YES)
@@ -296,9 +368,19 @@ cb_send_signal (GtkMenuItem *mi, gpointer user_data)
 			_("An error was encountered by sending a signal to the PID %d. "
 			"It is likely you don't have the required privileges."), pid);
 		gtk_window_set_title (GTK_WINDOW (dialog), _("Task Manager"));
+		gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 	}
+
+	if (xtm_signal == XTM_SIGNAL_TERMINATE || xtm_signal == XTM_SIGNAL_KILL)
+	{
+		GtkTreeSelection *selection;
+		GtkWidget *treeview;
+		treeview = g_object_get_data (G_OBJECT (mi), "treeview");
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
+		gtk_tree_selection_unselect_all (selection);
+	}
 }
 
 static void
@@ -314,18 +396,25 @@ cb_set_priority (GtkMenuItem *mi, gpointer user_data)
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), _("An error was encountered by setting a priority to the PID %d. "
 			"It is likely you don't have the required privileges."), pid);
 		gtk_window_set_title (GTK_WINDOW (dialog), _("Task Manager"));
+		gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
 	}
 }
 
 static GtkWidget *
-build_context_menu (guint pid)
+build_context_menu (XtmProcessTreeView *treeview, guint pid)
 {
 	GtkWidget *menu, *menu_priority, *mi;
 
 	menu = gtk_menu_new ();
 
+	mi = gtk_menu_item_new_with_label (_("Terminate"));
+	g_object_set_data (G_OBJECT (mi), "pid", GUINT_TO_POINTER (pid));
+	g_object_set_data (G_OBJECT (mi), "treeview", treeview);
+	gtk_container_add (GTK_CONTAINER (menu), mi);
+	g_signal_connect (mi, "activate", G_CALLBACK (cb_send_signal), GINT_TO_POINTER (XTM_SIGNAL_TERMINATE));
+
 	if (!pid_is_sleeping (pid))
 	{
 		mi = gtk_menu_item_new_with_label (_("Stop"));
@@ -346,11 +435,6 @@ build_context_menu (guint pid)
 	gtk_container_add (GTK_CONTAINER (menu), mi);
 	g_signal_connect (mi, "activate", G_CALLBACK (cb_send_signal), GINT_TO_POINTER (XTM_SIGNAL_KILL));
 
-	mi = gtk_menu_item_new_with_label (_("Terminate"));
-	g_object_set_data (G_OBJECT (mi), "pid", GUINT_TO_POINTER (pid));
-	gtk_container_add (GTK_CONTAINER (menu), mi);
-	g_signal_connect (mi, "activate", G_CALLBACK (cb_send_signal), GINT_TO_POINTER (XTM_SIGNAL_TERMINATE));
-
 	menu_priority = gtk_menu_new ();
 
 	mi = gtk_menu_item_new_with_label (_("Very low"));
@@ -387,18 +471,39 @@ build_context_menu (guint pid)
 	return menu;
 }
 
+static void
+position_menu (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, XtmProcessTreeView *treeview)
+{
+	gdk_window_get_origin (gtk_tree_view_get_bin_window (GTK_TREE_VIEW (treeview)), x, y);
+	*x += 5;
+	*y += 5;
+	*push_in = TRUE;
+}
+
+static void
+popup_menu (XtmProcessTreeView *treeview, guint pid, guint activate_time, gboolean at_pointer_position)
+{
+	static GtkWidget *menu = NULL;
+	GtkMenuPositionFunc position_func = NULL;
+
+	if (at_pointer_position == FALSE)
+		position_func = (GtkMenuPositionFunc)position_menu;
+
+	if (menu != NULL)
+		gtk_widget_destroy (menu);
+
+	menu = build_context_menu (treeview, pid);
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, position_func, treeview, 1, activate_time);
+}
+
 static gboolean
 treeview_clicked (XtmProcessTreeView *treeview, GdkEventButton *event)
 {
-	static GtkWidget *menu = NULL;
 	guint pid;
 
 	if (event->button != 3)
 		return FALSE;
 
-	if (menu != NULL)
-		gtk_widget_destroy (menu);
-
 	{
 		GtkTreeModel *model;
 		GtkTreeSelection *selection;
@@ -422,8 +527,33 @@ treeview_clicked (XtmProcessTreeView *treeview, GdkEventButton *event)
 #endif
 	}
 
-	menu = build_context_menu (pid);
-	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, event->button, event->time);
+	popup_menu (treeview, pid, event->time, TRUE);
+
+	return TRUE;
+}
+
+static gboolean
+treeview_key_pressed (XtmProcessTreeView *treeview, GdkEventKey *event)
+{
+	guint pid;
+
+	if (event->keyval != GDK_Menu)
+		return FALSE;
+
+	{
+		GtkTreeModel *model;
+		GtkTreeSelection *selection;
+		GtkTreeIter iter;
+
+		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
+
+		if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+			return FALSE;
+
+		gtk_tree_model_get (model, &iter, XTM_PTV_COLUMN_PID, &pid, -1);
+	}
+
+	popup_menu (treeview, pid, event->time, FALSE);
 
 	return TRUE;
 }
@@ -444,7 +574,7 @@ column_clicked (GtkTreeViewColumn *column, XtmProcessTreeView *treeview)
 		gtk_tree_view_column_set_sort_indicator (treeview->sort_column, FALSE);
 		gtk_tree_view_column_set_sort_indicator (column, TRUE);
 		sort_column_id = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (column), "sort-column-id"));
-		sort_type = GTK_SORT_ASCENDING;
+		sort_type = GTK_SORT_DESCENDING;
 	}
 	else
 	{
@@ -519,6 +649,13 @@ settings_changed (GObject *object, GParamSpec *pspec, XtmProcessTreeView *treevi
 		g_object_get (object, pspec->name, &visible, NULL);
 		gtk_tree_view_column_set_visible (gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), treeview->columns_positions[column_id]), visible);
 	}
+	else if (!g_strcmp0 (pspec->name, "show-application-icons"))
+	{
+		GtkTreeViewColumn *column;
+		column = gtk_tree_view_get_column (GTK_TREE_VIEW (treeview), treeview->columns_positions[COLUMN_COMMAND]);
+		gtk_tree_view_column_clear (column);
+		column_task_pack_cells (treeview, column);
+	}
 	else if (!g_strcmp0 (pspec->name, "show-all-processes"))
 	{
 		g_object_get (object, pspec->name, &treeview->show_all_processes_cached, NULL);
